/***************************************************************

   pidtst.c

   Tests the PID control

   Author:  Wade Cobb

   Date:    August 3, 1993

****************************************************************/
#define  STRICT
#include <windows.h>
#include <stdlib.h>
#include "pid.h"
#include "pidtst.h"

//These are globals for convenience only.
HINSTANCE   hInstMain;

/***************************************************************

                  Main Window Procedure

****************************************************************/
int PASCAL WinMain (HINSTANCE hInstance, 
                    HINSTANCE hPrevInstance,
                    LPSTR     lpszCmdLine, 
                    int       nCmdShow)
{
   WNDCLASS    wc;
   HWND        hWnd;
   MSG         msg;

   if (!hPrevInstance)
   {
       wc.style          = CS_HREDRAW | CS_VREDRAW;
       wc.lpfnWndProc    = pidtstWndProc;
       wc.cbClsExtra     = 0;
       wc.cbWndExtra     = 0;
       wc.hInstance      = hInstance;
       wc.hIcon          = LoadIcon(hInstance, "pidtstIcon");
       wc.hCursor        = LoadCursor(NULL, IDC_ARROW);
       wc.hbrBackground  = (HBRUSH)(COLOR_WINDOW + 1);
       wc.lpszMenuName   = "pidtstMenu";
       wc.lpszClassName  = "pidtst";


       if (!RegisterClass(&wc))
           return FALSE;
   }

   hInstMain=hInstance;

      //Create and show main window.
   hWnd=CreateWindow("pidtst", "PID Test",
                     WS_MINIMIZEBOX | WS_OVERLAPPEDWINDOW,
                     35, 35, 350, 340,
                     NULL, NULL, hInstance, NULL);

   ShowWindow(hWnd, nCmdShow);
   UpdateWindow(hWnd);

     //Force the PID dll to load
   registerPID();

   while (GetMessage(&msg, NULL, 0, 0))
   {
       TranslateMessage(&msg);
       DispatchMessage(&msg);
   }

   return msg.wParam;
}



/*
 * pidtstWndProc - the main window proc
 *
 *
 */

long FAR PASCAL pidtstWndProc(HWND hWnd, UINT iMessage,
			       WPARAM wParam, LPARAM lParam)
{
    switch (iMessage)
    {
        case WM_CREATE:
            break;

        case WM_DESTROY:
            PostQuitMessage(0);
            break;

        case WM_COMMAND:
            switch (wParam)
            {
               case IDM_TEST:

                     //Create the dialog box with the PID control in it
                  DialogBox(hInstMain,"DEMODLG",hWnd,(DLGPROC)PIDDlgProc);
                  break;

               case IDM_EXIT:
                  PostMessage(hWnd, WM_CLOSE, 0, 0L);
                  break;
            }
            break;

        default:
            return (DefWindowProc(hWnd, iMessage, wParam, lParam));
    }

    return 0L;
}

/********************************************************************

      PIDDlgProc

      Dialog box proc for the PID demo application.

*********************************************************************/
BOOL FAR PASCAL PIDDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   LPARAM   l1;
   char     szVal[20];
   float    fVal,mVal,oVal;

   switch (message)
   {
      case WM_INITDIALOG:

            //Set the colors of the slider arrows
         SendDlgItemMessage(hWnd,IDC_PIDCONTROL,PIDM_SETSLIDERCOLOR,
                     SLIDER_SETPOINT_ARROW,(LPARAM)0x00FF0000);
         SendDlgItemMessage(hWnd,IDC_PIDCONTROL,PIDM_SETSLIDERCOLOR,
                     SLIDER_MEASURED_ARROW,(LPARAM)0x0000FF00);
         SendDlgItemMessage(hWnd,IDC_PIDCONTROL,PIDM_SETSLIDERCOLOR,
                     SLIDER_OUTPUT_ARROW,(LPARAM)0x000000FF);

            //Set the poll period to three seconds
         (float)l1=(float)3.0;
         SendDlgItemMessage(hWnd,IDC_PIDCONTROL,PIDM_SETPOLLPERIOD,
                     0,l1);
         SetDlgItemText(hWnd,IDC_POLLPERIOD,"3.0");

            //Set the controller range
         (float)l1=(float)100.0;
         SendDlgItemMessage(hWnd,IDC_PIDCONTROL,PIDM_SETSETPOINTMAX,
                     0,(LPARAM)l1);
         SetDlgItemText(hWnd,IDC_RANGEMAX,"100.0");

         (float)l1=(float)0.0;
         SendDlgItemMessage(hWnd,IDC_PIDCONTROL,PIDM_SETSETPOINTMIN,
                     0,(LPARAM)l1);
         SetDlgItemText(hWnd,IDC_RANGEMIN,"0.0");

            //Set the output direction to normal
         SendDlgItemMessage(hWnd,IDC_PIDCONTROL,PIDM_SETOUTPUTDIRECTION,
                     OUTPUTNORMAL,0L);

            //Set the output mode to automatic
         SendDlgItemMessage(hWnd,IDC_PIDCONTROL,PIDM_SETOUTPUTMODE,
                     OUTPUTAUTOMATIC,0L);
         SendDlgItemMessage(hWnd,IDC_AUTOMATIC, BM_SETCHECK,1,0L);
         
            //Set the gain to 50
         (float)l1=(float)50;
         SendDlgItemMessage(hWnd,IDC_PIDCONTROL,PIDM_SETGAIN,
                     0,l1);
         SetDlgItemText(hWnd,IDC_GAIN,"50.0");

            //Set the reset to 2
         (float)l1=(float)2.0;
         SendDlgItemMessage(hWnd,IDC_PIDCONTROL,PIDM_SETRESET,
                     0,l1);
         SetDlgItemText(hWnd,IDC_RESET,"2.0");

            //Set the preact to .2
         (float)l1=(float).2;
         SendDlgItemMessage(hWnd,IDC_PIDCONTROL,PIDM_SETPREACT,
                     0,(LPARAM)l1);
         SetDlgItemText(hWnd,IDC_PREACT,".2");

            //Set up a timer for doing the simulation - it will go off
            // every two seconds
         SetTimer(hWnd,1,2000,NULL);
         break;

      case WM_TIMER:

            //Adjust the measured value based on its current value and
            // the output value

               // Get the output value
         SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                            PIDM_GETOUTPUT,0,(LPARAM)&oVal);

               // Get the current measured value
         SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                            PIDM_GETMEASURED,0,(LPARAM)&mVal);

            //If the output is less than 10 (very arbitrary), decrease
            // the measured value
         if (oVal<10)
         {
            mVal-=(10-oVal)/10;      
         }
         else
         {
               //Else increase it
            mVal+=(oVal-10)/10;
         }
         (float)l1=mVal;
         SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                            PIDM_SETMEASURED,0,
                            l1);
         break;

      case WM_CTLCOLOR:

            //Set the color of the PID controller to gray
         if ( (HWND)LOWORD(lParam)==GetDlgItem(hWnd,IDC_PIDCONTROL))
         {
            SetBkColor((HDC)wParam,RGB(192,192,192));
         }
         return (BOOL)(WORD)(HBRUSH)GetStockObject(WHITE_BRUSH);

      case WM_COMMAND:
         switch (wParam)
         {
            case IDC_PIDCONTROL:

                  //Handle the notification messages
               switch(HIWORD(lParam))
               {
                  case PIDN_SETPOINTCHANGE:

                        //Get the new setpoint and update the text control
                     SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                                    PIDM_GETSETPOINT,0,(LPARAM)&fVal);

                     _gcvt((double)fVal,5,szVal);
                     SetDlgItemText(hWnd,IDC_SETPOINT,szVal);
                     break;

                  case PIDN_OUTPUTCHANGE:

                        //Get the new output and update the text control
                     SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                                    PIDM_GETOUTPUT,0,(LPARAM)&fVal);

                     _gcvt((double)fVal,5,szVal);
                     SetDlgItemText(hWnd,IDC_OUTPUT,szVal);
                     break;

                  case PIDN_MEASUREDCHANGE:

                        //Get the new output and update the text control
                     SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                                    PIDM_GETMEASURED,0,(LPARAM)&fVal);

                     _gcvt((double)fVal,5,szVal);
                     SetDlgItemText(hWnd,IDC_MEASURED,szVal);
                     break;
               }
               break;

            case IDC_REVERSE:
               if (SendDlgItemMessage(hWnd,IDC_REVERSE,BM_GETCHECK,0,0L))
               {
                  SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                                    PIDM_SETOUTPUTDIRECTION,OUTPUTREVERSE,0L);
               }
               else
               {
                  SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                                    PIDM_SETOUTPUTDIRECTION,OUTPUTNORMAL,0L);
               }
               break;

            case IDC_AUTOMATIC:
               if (SendDlgItemMessage(hWnd,IDC_AUTOMATIC,BM_GETCHECK,0,0L))
               {
                  SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                                    PIDM_SETOUTPUTMODE,OUTPUTAUTOMATIC,0L);
               }
               else
               {
                  SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                                    PIDM_SETOUTPUTMODE,OUTPUTMANUAL,0L);
               }
               break;

            case IDC_RUN:
               SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                                  PIDM_SETACTION,ACTION_RUN,0L);
               break;

            case IDC_STOP:
               SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                                  PIDM_SETACTION,ACTION_STOP,0L);
               break;

            case IDC_RESETCONTROLLER:
               SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                                  PIDM_SETACTION,ACTION_RESET,0L);
               break;

            case IDC_GAIN:
               if (HIWORD(lParam)==EN_CHANGE)
               {
                  GetDlgItemText(hWnd,IDC_GAIN,szVal,sizeof(szVal));
                  fVal=(float)atof(szVal);
                  (float)l1=fVal;
                  SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                                     PIDM_SETGAIN,0,
                                     l1);
               }
               break;

            case IDC_RESET:
               if (HIWORD(lParam)==EN_CHANGE)
               {
                  GetDlgItemText(hWnd,IDC_RESET,szVal,sizeof(szVal));
                  fVal=(float)atof(szVal);
                  (float)l1=fVal;
                  SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                                     PIDM_SETRESET,0,
                                     l1);
               }
               break;

            case IDC_PREACT:
               if (HIWORD(lParam)==EN_CHANGE)
               {
                  GetDlgItemText(hWnd,IDC_PREACT,szVal,sizeof(szVal));
                  fVal=(float)atof(szVal);
                  (float)l1=fVal;
                  SendDlgItemMessage(hWnd,IDC_PIDCONTROL,
                                     PIDM_SETPREACT,0,
                                     l1);
               }
               break;

            case IDOK:
            case IDCANCEL:
               EndDialog(hWnd,FALSE);
               break;
         }
         break;

      default:
         return FALSE;
   }
   return TRUE;
}
